#include <stdio.h>

#include <glib.h>
#include <glib-object.h>

#include <misc/utils.h>
#include <qmail/qmail.h>
#include <vdomain/vdomain.h>

static gboolean quiet = FALSE;
static gboolean show_aliases = FALSE;
static gboolean show_users = FALSE;

static const GOptionEntry entries[] = {
	{ "aliases",		'a', 0, G_OPTION_ARG_NONE,		&show_aliases,	"Show only accounts without a mailbox",		NULL	},
	{ "quiet",		0,   0, G_OPTION_ARG_NONE,		&quiet,		"Suppress all status messages",			NULL	},
	{ "users",		'u', 0, G_OPTION_ARG_NONE,		&show_users,	"Show only accounts with a mailbox",		NULL	},
	{ NULL }
};

static void dump_all(VDomain *domain)
{
	VDomainEntry *entry;
	VDomainIter *iter;

	iter = v_domain_iter_new(domain);
	while((entry = v_domain_iter_next(iter, NULL)) != NULL) {
		if ((show_aliases && !entry->maildir) ||
		    (show_users && entry->maildir)) {
			gchar *dump = v_domain_entry_dump(entry);

			printf("%s\n", dump);
			g_free(dump);
		}
		g_object_unref(entry);
	}
	v_domain_iter_free(iter);
}

static void dump_list(VDomain *domain, gchar **list)
{
	for(; *list; list++) {
		VDomainEntry *entry;
		gchar *tmp;
		
		tmp = g_strstrip(g_ascii_strdown(*list, -1));
		entry = v_domain_get_entry(domain, tmp, NULL);
		if (!entry) {
			if (!quiet)
				g_warning(_("Virtual user `%s' unknown."),
				          tmp);
			g_free(tmp);
			continue;
		}
		g_free(tmp);

		if ((show_aliases && !entry->maildir) ||
		    (show_users && entry->maildir)) {
			gchar *dump = v_domain_entry_dump(entry);

			printf("%s\n", dump);
			g_free(dump);
		}
		g_object_unref(entry);
	}
}

int main(int argc, char **argv)
{
	VDomain *domain;
	gint retval = 1;

	v_init(&argc, &argv, entries, NULL, _("[USERNAME...]"),
	       _("Lists the members of a virtual domain"));
	               
	if (!show_aliases && !show_users)
		show_aliases = show_users = TRUE;

	domain = v_domain_new(NULL, NULL, FALSE, NULL);
	if (!domain) {
		if (!quiet)
			fprintf(stderr, _("%s: Could not open virtual users "
			                  "database.\n"), g_get_prgname());
		goto out;
	}

	printf(_("User Mailbox Aliases\n"));
	if (argc > 1)
		dump_list(domain,&argv[1]);
	else
		dump_all(domain);

	retval = 0;
out:
	if (domain)
		g_object_unref(domain);

	return retval;
}
