#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include <glib.h>
#include <glib-object.h>

#include <misc/utils.h>
#include <qmail/qmail.h>
#include <vdomain/vdomain.h>

static gboolean quiet = FALSE;
static gboolean no_maildir = FALSE;

static const GOptionEntry entries[] = {
	{ "no-mailbox",		'D', 0, G_OPTION_ARG_NONE,		&no_maildir,	"Do not delete users that have a mailbox",	NULL	},
	{ "quiet",		0,   0, G_OPTION_ARG_NONE,		&quiet,		"Suppress all status messages",			NULL	},
	{ NULL }
};

int main(int argc, char **argv)
{
	VDomainEntry *entry = NULL;
	VDomain *domain = NULL;
	GSList *users = NULL, *next;
	gchar *tmp;
	gint retval = 1;
	int i;

	v_init(&argc, &argv, entries, NULL, _("USER1 [USER2...]"),
	       _("Delete users from a virtual domain"));

	if (argc < 2)
		v_init_error(_("Too few command-line arguments"));

	domain = v_domain_new(NULL, NULL, FALSE, NULL);
	if (!domain) {
		fprintf(stderr,
		        _("%s: Could not initialize domain handler.\n"),
		        g_get_prgname());
		goto out;
	}

	for(i = 1; i < argc; i++) {
		tmp = g_strstrip(g_ascii_strdown(argv[i], -1));
		entry = v_domain_get_entry(domain, tmp, NULL);
		if (!entry && !quiet) {
			fprintf(stderr,
			        _("%s: Error removing the virtual user "
			          "`%s': %s.\n"), g_get_prgname(), tmp,
			        _("User does not exist"));
			g_free(tmp);
			goto out;
		}
		g_free(tmp);

		if (entry->maildir && no_maildir && !quiet) {
			fprintf(stderr,
			        _("%s: Error removing the virtual user "
			          "`%s': %s.\n"), g_get_prgname(),
			        entry->name, _("User has a maildir"));
			goto out;
		}

		v_domain_remove_entry(domain, entry->name, NULL);

		users = g_slist_prepend(users, entry);
		entry = NULL;
	}

	if (!v_domain_commit_changes(domain, NULL)) {
		if (!quiet)
			fprintf(stderr, _("%s: Error commiting changes to "
			                  "the database.\n"), g_get_prgname());
		goto out;
	}

	users = g_slist_reverse(users);

	for(next = users; next; next = users->next) {
		VMaildir *maildir;

		entry = V_DOMAIN_ENTRY(next->data);
		maildir = entry->maildir ? v_maildir_new(entry->maildir, NULL)
		                         : NULL;
		if (!maildir)
			continue;

		if (!v_maildir_delete(maildir, NULL) && !quiet)
			fprintf(stderr, _("%s: Error removing the maildir "
			                  " for user `%s'.\n"),
			        g_get_prgname(), entry->name);

		g_object_unref(maildir);
	}
	entry = NULL;

	if (!quiet)
		for(next = users; next; next = users->next)
			printf(_("%s: user `%s' successfully removed.\n"),
			       g_get_prgname(),
			       V_DOMAIN_ENTRY(next->data)->name);

	retval = 0;
out:
	if (entry)
		g_object_unref(entry);
	while(users) {
		next = users->next;
		g_object_unref(users->data);
		g_slist_free_1(users);
		users = next;
	}
	if (domain)
		g_object_unref(domain);
	return retval;
}
